package com.boe.zxy.interfacePlat.job;

import com.boe.zxy.interfacePlat.service.AsyncRemoteDataService;
import com.boe.zxy.interfacePlat.service.MateiralShipService;
import com.boe.zxy.interfacePlat.service.QMSService;
import lombok.extern.slf4j.Slf4j;
import org.apache.xmlbeans.impl.xb.xsdschema.Public;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * @project interfacePlat
 * @description
 * @author 10539622
 * @date 2023/8/27 18:15
 * @version 1.0
 */
@Component
public class JobService {

    private static final Logger log = LoggerFactory.getLogger(JobService.class);

    @Autowired
    AsyncRemoteDataService asyncRemoteDataService;
    @Autowired
    MateiralShipService mateiralShipService;

    @Autowired
    QMSService qMSService;

    // 使用Scheduled注解定义一个定时任务方法
    // cron表达式"0 0 8 ? * MON"表示每周一上午8点触发该方法
    @Scheduled(cron = "0 0 8 ? * MON")
    public void SendAgeInfpMail() throws Exception {
        // 记录定时任务开始执行的日志
        log.info("开始调用寿命邮件定时器");

        // 调用远程数据服务的异步方法，发送寿命信息邮件
        asyncRemoteDataService.SendAgeInfpMail();

        // 记录定时任务执行结束的日志
        log.info("结束调用寿命邮件定时器");
    }



    // 定义一个定时任务方法，使用Cron表达式调度
    @Scheduled(cron = "0 11 18 * *  ?")
    public void SendQms () {
        // 记录任务开始日志
        log.info("开始品质任务发送");

        // 调用QMSService的sendQms方法执行具体任务
        qMSService.sendQms();

        // 记录任务结束日志
        log.info("结束品质任务发送");
    }


    @Scheduled(cron = "0 0 1 * *  ?")
    public void SendQmToERP () {
        // 记录任务开始日志
        log.info("开始品质任务发送");

        // 调用QMSService的sendQms方法执行具体任务
        qMSService.sendQmToERP();

        // 记录任务结束日志
        log.info("结束品质任务发送");
    }


}
