package com.boe.zxy.interfacePlat.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.boe.zxy.interfacePlat.entity.BeanMateiralShip;
import com.boe.zxy.interfacePlat.mapper.sdk.MateiralShip;
import com.boe.zxy.interfacePlat.util.http.FMHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 10539622
 * @version 1.0
 * @project interfacePlat
 * @description 自动领料服务
 * @date 2024/2/27 15:09:17
 */
@Slf4j
@Service
public class MateiralShipService {

    @Autowired
    MateiralShip mateiralShip;

    @Autowired
    private static FMHttpUtil fmHttpUtil;

    private static String baseUrl = "http://202.101.116.87:8022";
    String url = "";

    static {
        if (fmHttpUtil == null) {
            fmHttpUtil = new FMHttpUtil();
        }
    }

    @Transactional
    public void MateiralShip () throws Exception {
        // 查询需要过账 的领料单
        List<BeanMateiralShip> listShip = mateiralShip.getListShip();

        for (BeanMateiralShip b: listShip) {
            //查询领料单明细
            List<BeanMateiralShip> shipInfo = mateiralShip.getShipInfo(b.getTwh_mesdjbh());

            JSONObject STK_MisDelivery = IssueDoc_Create_OutStock(shipInfo, "mes", "admin");
            url = "/ierp/kapi/v2/twh/pdm/imMdcMftproorderSave";

            String doExchange = fmHttpUtil.doExchange(baseUrl + url, STK_MisDelivery);

            org.json.JSONObject jsonObject = new org.json.JSONObject(doExchange);

            boolean isSuccess = (boolean) jsonObject.get("status");
            String message = "";
            if (!isSuccess) {
                // 插入错误日志
                message = jsonObject.get("message").toString();
                BeanMateiralShip beanMateiralShip = new BeanMateiralShip();
                beanMateiralShip.setShipRequestname(b.getTwh_mesdjbh());
                beanMateiralShip.setNote(message);
                mateiralShip.updateFailInfo(beanMateiralShip);
                continue;
            } else {
                Map<String,Object> map = new HashMap<String,Object>();
//
//                #{siteName, mode=IN, jdbcType=VARCHAR},
//            #{twh_mesdjbh, mode=IN, jdbcType=VARCHAR},
//            #{userId, mode=IN, jdbcType=VARCHAR},
//            #{v_result, mode=OUT, jdbcType=VARCHAR},
                map.put("siteName","FM");
                map.put("twh_mesdjbh",b.getTwh_mesdjbh());
                map.put("userId","admin");
                map.put("v_result","");
                mateiralShip.callAuditCall(map);
            }


        }


    }


    private static JSONObject IssueDoc_Create_OutStock(List<BeanMateiralShip>  shipResult, String systemtype, String eventUser) {




        JSONObject all = new JSONObject(true);

        JSONArray arr = new JSONArray();

        JSONObject body = new JSONObject(true);
        //		twh_mesdjbh	String	是	MES单据编号	1	"HpMiT"
        body.put("twh_mesdjbh", shipResult.get(0).getTwh_mesdjbh());
        if ("erp".equals(systemtype)) {

            body.put("billno", shipResult.get(0).getTwh_mesdjbh());
        }
        //		biztime	Date	是	业务日期	1	"2023-03-16"
        //
        body.put("biztime", new SimpleDateFormat("yyyy-MM-dd").format(new Date()));
        //		comment	String	否	备注	1	"e0t1P"
        body.put("comment", null);
        //		bookdate	Date	否	记账日期	1	"2023-03-16"
        body.put("bookdate", new SimpleDateFormat("yyyy-MM-dd").format(new Date()));
        //		requser_number	String	是	领用人.工号	1	"dfmvo"
        body.put("requser_number", shipResult.get(0).getRequser_number());
        //		operator_operatornumber	String	否	库管员.业务员编码	1	"SSULE"
        body.put("operator_operatornumber", "".equals(eventUser) ?  "00000001" : eventUser) ;
//   curl -i -X POST -H 'Content-Type:application/x-www-form-urlencoded;charset=UTF-8' -d '{"pobillcode": "CGDD.701.230814.014","company": "701"}' ''

        body.put("twh_sssyb", "00000019");
        //单据明细行
        JSONArray detailArr = new JSONArray();
        for (BeanMateiralShip map : shipResult) {
            JSONObject detail = new JSONObject(true);
            //			  productnum	String	否	物料明细.生产编号	2	"lsKyn"
            detail.put("productnum", null);
            //			  seq	Integer	否	物料明细.分录行号	2	40
            detail.put("seq", map.getSeq());
            //			  qty	Decimal	是	物料明细.数量	2	"945.70"
            detail.put("qty", map.getQty());
            //			  lotnumber	String	否	物料明细.批号	2	"UK8IC"
            detail.put("lotnumber", map.getLot_number());
            //			  entrycomment	String	否	物料明细.备注	2	"QPoi0"
            detail.put("entrycomment", map.getEntrycomment());
            //			  baseqty	Decimal	否	物料明细.基本数量	2	"691.52"
            detail.put("baseqty", map.getBaseqty());
            //			  producedate	Date	否	物料明细.生产日期	2	"2023-03-16"
            String bzq = map.getBzq() == null ? "N" : map.getBzq() ;// 是否启用保质期管理
            if ("Y".equals(bzq)) {
                detail.put("producedate", map.getProducedate());
                detail.put("expirydate", map.getExpirydate());
            }
            //			  baseunit_number	String	是	基本单位.编码	2	"K4ob0"
            detail.put("baseunit_number", map.getBaseunit_number());
            //			  unit_number	String	是	计量单位.编码	2	"zlVXO"
            detail.put("unit_number", map.getUnit_number());
            //			  warehouse_number	String	是	仓库.编码	2	"zq3cD"
            detail.put("warehouse_number", map.getWarehouse_number());
            //			  lot_number	String	是	批号主档.批号	2	"VzXNz"
            detail.put("lot_number", map.getLot_number());
            //			  material_number	String	否	物料编码.编码	2	"L22nw"
            detail.put("material_number", map.getMaterial_number());
            //			  twh_sydjdh	String	是	上游单据单号	2	"123456"
            detail.put("twh_sydjdh", map.getTwh_sydjdh());
            //			  twh_sydjhh	String	是	上游单据行号	2	"1"
            detail.put("twh_sydjhh", map.getTwh_sydjhh());

            detailArr.add(detail);
        }
        body.put("billentry", detailArr);

        arr.add(body);

        all.put("data", arr);
        return all;


    }

}
