package com.boe.zxy.interfacePlat.util.http;

/**
 * @Description TODO
 * @Author li
 * @Date 2024/12/23 14:37
 */
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.boe.zxy.interfacePlat.entity.BeanQMS;
import com.boe.zxy.interfacePlat.entity.BeanQMToERP;
import com.boe.zxy.interfacePlat.entity.ErpMessageLog;
import com.boe.zxy.interfacePlat.mapper.sdk.MakeLog;
import com.boe.zxy.interfacePlat.util.TimeStampUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Logger;

@Service
public class BatchProcessorToERPUtil {

    @Autowired
    MakeLog makeLog;
    private  static String  ERPUrl = "http://192.168.1.35:8079/service/ToNCServlet";
    // 日志记录器
    private static final Logger logger = Logger.getLogger(BatchProcessorToERPUtil.class.getName());

    // 假设这是你要调用的接口的实现类
    private  class ExternalService {
        public   void  processBatch(String s) throws DocumentException {
            // 这里是调用外部接口的逻辑
            // 例如：发送HTTP请求到另一个服务
            String httpPost = NCHttpUtil.httpPost(ERPUrl , s);
            // 模拟处理时间
            try {
                Thread.sleep(10000); // 假设处理需要1秒钟
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.severe("Batch processing interrupted");
            } finally {
                Document document1 = DocumentHelper.parseText(httpPost);
                org.dom4j.Element rootElement = document1.getRootElement();
                String rcode = rootElement.elementText("rcode");
                String rmsg = rootElement.elementText("rmsg");
                String billcode = rootElement.elementText("billcode");
                UUID uuid = UUID.randomUUID();
                ErpMessageLog erplog = new ErpMessageLog();
                erplog.setEventUser("WmsToErp");
                erplog.setServerName("WmsToErp");
                erplog.setEventName("AutoWareTranfer");
                erplog.setMessageId(UUID.randomUUID().toString());
                erplog.setInterfaceTime(TimeStampUtil.getCurrentTime(TimeStampUtil.FORMAT_DEFAULT));
                erplog.setMessageId(UUID.randomUUID().toString());
                erplog.setSendMsg2(s);
                erplog.setSendMsg(s);
                erplog.setReturnMsg2(httpPost);
                erplog.setResultCode(rcode);
//                new MessageLogUtil().writeMessageLog(erplog);
                int i = makeLog.insertLog(erplog);
            }
        }
    }

    // 线程任务类
    private  class BatchTask implements Runnable {
        private  List<BeanQMToERP> batch;

        public BatchTask(List<BeanQMToERP> batch, ExternalService externalService) {
            this.batch = batch;
        }

        @Override
        public void run() {
            String s = null;
            try {
                s = makeERPLocationMoveNoIovoiceXML(batch, "004557", UUID.randomUUID().toString());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
            try {
                new ExternalService().processBatch(s);
            } catch (DocumentException e) {
                throw new RuntimeException(e);
            }
        }
    }

    // 批处理的主方法
    public  void processDataInBatches(List<BeanQMToERP> data, int batchSize, ExternalService externalService) {
        // 创建一个固定大小的线程池
        ExecutorService executorService = Executors.newFixedThreadPool(10);

        // 计算需要多少个批次
        int numberOfBatches = (int) Math.ceil((double) data.size() / batchSize);

        for (int i = 0; i < numberOfBatches; i++) {
            int start = i * batchSize;
            int end = Math.min(start + batchSize, data.size());
            List<BeanQMToERP> batch = data.subList(start, end);

            // 提交任务到线程池
            executorService.submit(new BatchTask(batch, externalService));
        }

        // 关闭线程池，并等待所有任务完成
        executorService.shutdown();
        while (!executorService.isTerminated()) {
            // 等待所有任务完成
        }

        logger.info("All batches processed");
    }

    // 组装发送QMS的json
    private String makeERPLocationMoveNoIovoiceXML(List<BeanQMToERP> list, String user, String opCode) throws Exception {
        if (list == null || list.size() < 1) {
            return "";
        }
//	    <billtype>4K</billtype>
//	    <isaudit>Y</isaudit>
//	    <opcode></opcode>
//	    <org>101</org>
//	    <iotype>4K-Cxx-07</iotype>
//	    <instore>10104065</instore>
//	    <outstore>10102025</outstore>
//	    <shouldindate />
//	    <shouldoutdate />
//	    <billtime>2023-12-20</billtime>

        //先保存成json再转xml
        ObjectMapper objectMapper = new ObjectMapper();
        com.fasterxml.jackson.dataformat.xml.XmlMapper xmlMapper = new com.fasterxml.jackson.dataformat.xml.XmlMapper();
        JSONObject bill = new JSONObject(true);
        JSONObject billBody = new JSONObject(true);
        billBody.put("billtype", list.get(0).getBilltype());
        billBody.put("isaudit", "Y");
        billBody.put("opcode", opCode);
        billBody.put("org", list.get(0).getOrg());
        billBody.put("instore", list.get(0).getInstore());
        billBody.put("outstore", list.get(0).getOutstore());
        billBody.put("iotype", list.get(0).getIotype());
        billBody.put("billtime", list.get(0).getBilltime());
        billBody.put("shouldindate", list.get(0).getBilltime());
        billBody.put("shouldoutdate", list.get(0).getBilltime());
        billBody.put("manager", user);
        billBody.put("maker", user);

        JSONArray jsonArray = new JSONArray();
        JSONArray itemH = new JSONArray();
        JSONObject detail = new JSONObject(true);
        for (BeanQMToERP beanQMToERP : list) {
//            <mat>401120444</mat>
//            <matversion>1</matversion>
//            <unit>11</unit>
//            <astunit>11</astunit>
//            <changerate>1.00/1.00</changerate>
//            <shouldastnum>708400000</shouldastnum>
//            <shouldnum>708400000</shouldnum>
//            <proddate>2022-04-02</proddate>
//            <expdate>2022-09-29</expdate>
//            <batchcode>2204020001_15645</batchcode>
//            <outloc>202500060</outloc>
//            <inloc>406500003</inloc>
//            <jd>S</jd>
//            <sdk>wxb44DA8B16EE7448B3B</sdk>
//        </item>
            JSONObject itemD = new JSONObject(true);
            itemD.put("mat", beanQMToERP.getMat());
            itemD.put("matversion",  beanQMToERP.getMatversion());
            itemD.put("unit", beanQMToERP.getUnit());
            itemD.put("astunit", beanQMToERP.getAstunit());
            itemD.put("changerate", beanQMToERP.getChangerate());
            itemD.put("batchcode", beanQMToERP.getBatchcode());
            itemD.put("expdate", beanQMToERP.getExpdate());
            itemD.put("proddate", beanQMToERP.getProddate());
            itemD.put("shouldastnum", beanQMToERP.getShouldastnum());
            itemD.put("shouldnum", beanQMToERP.getShouldnum());
            itemD.put("outloc", beanQMToERP.getOutloc());
            itemD.put("inloc", beanQMToERP.getInloc());
            itemD.put("jd", beanQMToERP.getJd());
            itemD.put("sdk", beanQMToERP.getSdk());
            itemH.add(itemD);
        }
        detail.put("item", itemH);
        billBody.put("detail", detail);
        bill.put("bill", billBody);
        Object readValue = objectMapper.readValue(bill.toJSONString(), Object.class);
        String writeValueAsString = xmlMapper.writeValueAsString(readValue);
        return writeValueAsString.replace("<LinkedHashMap>", "").replace("</LinkedHashMap>", "");
    }





}