package com.boe.zxy.interfacePlat.util.http;

/**
 * @Description TODO
 * @Author li
 * @Date 2024/12/23 14:37
 */
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.boe.zxy.interfacePlat.entity.BeanQMS;
import com.boe.zxy.interfacePlat.entity.ErpMessageLog;
import com.boe.zxy.interfacePlat.mapper.sdk.MakeLog;
import com.boe.zxy.interfacePlat.util.TimeStampUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Logger;

@Service
public class BatchProcessorToQMSUtil {

    @Autowired
    MakeLog makeLog;
    private  static String  qmsUrl = "http://192.168.1.72";
    // 日志记录器
    private static final Logger logger = Logger.getLogger(BatchProcessorToQMSUtil.class.getName());

    // 假设这是你要调用的接口的实现类
    private  class ExternalService {
        public   void  processBatch(String s) {
            // 这里是调用外部接口的逻辑
            // 例如：发送HTTP请求到另一个服务
            String httpPost = NCHttpUtil.httpPost(qmsUrl + "/apitest/sdkqms/Common/wms/triggerInspect", s);
            // 模拟处理时间
            try {
                Thread.sleep(10000); // 假设处理需要1秒钟
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.severe("Batch processing interrupted");
            } finally {
                org.json.JSONObject jsonObject = new org.json.JSONObject(httpPost);
                String code = jsonObject.getString("code");
                ErpMessageLog erplog = new ErpMessageLog();
                erplog.setEventUser("");
                erplog.setServerName("WmsToQMSGQ");
                erplog.setEventName("WmsToQMSGQ");
                erplog.setInterfaceTime(TimeStampUtil.getCurrentTime(TimeStampUtil.FORMAT_DEFAULT));
                erplog.setTimeKey(TimeStampUtil.getCurrentEventTimeKey());
                erplog.setMessageId(UUID.randomUUID().toString());
                erplog.setSendMsg(s);
                erplog.setReturnMsg2(httpPost);
                erplog.setResultCode(code);
//                new MessageLogUtil().writeMessageLog(erplog);
                int i = makeLog.insertLog(erplog);
            }
        }
    }

    // 线程任务类
    private  class BatchTask implements Runnable {
        private  List<BeanQMS> batch;

        public BatchTask(List<BeanQMS> batch, ExternalService externalService) {
            this.batch = batch;
        }

        @Override
        public void run() {
            String s = MakeQMSSendJson(batch, "3", "");
            new ExternalService().processBatch(s);
        }
    }

    // 批处理的主方法
    public  void processDataInBatches(List<BeanQMS> data, int batchSize, ExternalService externalService) {
        // 创建一个固定大小的线程池
        ExecutorService executorService = Executors.newFixedThreadPool(10);

        // 计算需要多少个批次
        int numberOfBatches = (int) Math.ceil((double) data.size() / batchSize);

        for (int i = 0; i < numberOfBatches; i++) {
            int start = i * batchSize;
            int end = Math.min(start + batchSize, data.size());
            List<BeanQMS> batch = data.subList(start, end);

            // 提交任务到线程池
            executorService.submit(new BatchTask(batch, externalService));
        }

        // 关闭线程池，并等待所有任务完成
        executorService.shutdown();
        while (!executorService.isTerminated()) {
            // 等待所有任务完成
        }

        logger.info("All batches processed");
    }

    // 组装发送QMS的json
    public  String MakeQMSSendJson(List<BeanQMS> list, String qmsType, String receiveActNo) {

        if (list == null || list.size() < 1) {
            return null;
        }

        JSONObject sendData = new JSONObject(true);

        JSONObject data = new JSONObject(true);
        sendData.put("data", data);
        JSONArray infoArray = new JSONArray();

        data.put("inspectionScenario", qmsType);
        data.put("infoList", infoArray);
        for (BeanQMS mm : list) {
            JSONObject item = new JSONObject(true);

            item.put("commonNumber", receiveActNo);
            item.put("warehouse", mm.getERPLOCATION());
            item.put("deliveryDate", mm.getDeliveryDate());
            item.put("shippingDate", mm.getShippingDate());
            item.put("materialCode", mm.getMATERIALSPECNAME());
            item.put("materialName", mm.getDESC_CN());
            item.put("materialGroup", mm.getMATERIALGROUP());
            item.put("specification", mm.getWIDTH());
            item.put("modelNumber", mm.getMATERIALSPECNAME());
            item.put("businessUnit", mm.getBUSINESS_UNIT());
            item.put("quantity", mm.getQTY());
            item.put("unit", mm.getMATERIALUNIT());
            item.put("lot", mm.getCHARGE().split(","));
            item.put("supplierCode", mm.getSUPPLIERCODE());
            item.put("erpLocation", mm.getErpFactory());
            item.put("supplierName", mm.getSUPPLIERNAME());
            infoArray.add(item);
        }

        return sendData.toJSONString();

    }




}